import { Component, OnInit , ChangeDetectionStrategy , ChangeDetectorRef , Renderer2 , ElementRef, ViewChild , inject } from '@angular/core';
import {FormControl, FormGroup, FormsModule, ReactiveFormsModule , Validators} from '@angular/forms';
import {MatIconModule} from '@angular/material/icon';
import {MatDividerModule} from '@angular/material/divider';
import {MatButtonModule} from '@angular/material/button';
import {provideNativeDateAdapter , MatNativeDateModule , MAT_DATE_FORMATS } from '@angular/material/core';
import {DateRange, MatDatepickerModule, MatDateRangeInput  } from '@angular/material/datepicker';
import {MatFormFieldModule} from '@angular/material/form-field';
import {MatInputModule} from '@angular/material/input';
import {MatSelectModule} from '@angular/material/select';
import {MatButtonToggleModule} from '@angular/material/button-toggle';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {ConformidadHelper} from "./helper/conformidad.helper";
import {NgClass, NgForOf, NgIf, DatePipe, AsyncPipe, NgStyle} from "@angular/common";
import {MatCard, MatCardContent, MatCardFooter, MatCardHeader, MatCardTitle , MatCardModule} from '@angular/material/card';
import {Observable, from, min} from 'rxjs';
import {switchMap, map, startWith , debounceTime } from 'rxjs/operators';
//import {AsyncPipe} from '@angular/common';
import {MatAutocompleteModule} from '@angular/material/autocomplete';
import {MatTooltipModule} from '@angular/material/tooltip';
import {MatDialog, MatDialogModule} from '@angular/material/dialog';
import {modalRequerimientoComponent} from "./modalRequerimiento/modalRequerimiento.componente";
import {MatTableModule} from '@angular/material/table';
import { MatCell, MatCellDef, MatColumnDef, MatHeaderCell, MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef, MatTable } from "@angular/material/table";
import { SoloNumerosDirective } from '../../service/directivas_service/soloNumeros/solo-numeros.directive'; // Importa la directiva
import { MY_DATE_FORMATS } from "../../service/directivas_service/formatoFecha/date-format"; // Asegúrate de ajustar la ruta

interface tiposTrabajo {
  value: string;
  viewValue: string;
}

interface interRequerimientos {
  codigo: string;
  serie : string;
  numero: string;
}

interface interDetallePasaje {
  index: number;
  lugarOrigen: string;
  lugarDestino : string;
  monto: string;
  fecha: Date;
  isNew: boolean;
}

@Component({
  imports: [
    SoloNumerosDirective,
    MatDatepickerModule,
    MatNativeDateModule,
    MatFormFieldModule,
    FormsModule,
    ReactiveFormsModule,
    MatSelectModule,
    MatInputModule,
    MatButtonToggleModule,
    MatCheckboxModule,
    NgForOf,
    MatCard,
    MatCardContent,
    MatCardFooter,
    MatCardHeader,
    MatCardTitle,
    MatCardModule,
    MatButtonModule,
    MatDividerModule,
    MatIconModule,
    MatAutocompleteModule,
    AsyncPipe,
    MatTooltipModule,
    NgClass,
    MatCell,
    MatCellDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderRow,
    MatHeaderRowDef,
    MatRow,
    MatRowDef,
    MatTable,
    MatTableModule,
    NgIf,
    NgStyle,
  ],
  providers: [provideNativeDateAdapter(),DatePipe,  { provide: MAT_DATE_FORMATS, useValue: MY_DATE_FORMATS }  ],
  selector: 'app-conformidad',
  standalone: true,
  templateUrl: './conformidad.componente.html',
  styleUrls: ['./conformidad.componente.css'],
  //changeDetection: ChangeDetectionStrategy.OnPush,
})

export class ConformidadComponent implements OnInit {


  // VIEWCHILD
  @ViewChild('labelAdvertencia', { static: false }) labelAdvertencia!: ElementRef;
  @ViewChild('inputBusqueda', { static: false }) inputBusqueda!: ElementRef;
  // VIEWCHILD


  //INICIALIZACION DE LISTADO DE TIPOS DE TRABAJO
  listTipoTrabajo: tiposTrabajo[] = [
    {value: '1', viewValue: 'Instalaciòn-Revisión'},
    {value: '2', viewValue: 'Instalaciòn-Ampliación'},
    {value: '3', viewValue: 'Mantenimiento-Reparación'},
    {value: '4', viewValue: 'Mantenimiento-Remodelación'},
    {value: '5', viewValue: 'Reposición'},
    {value: '6', viewValue: 'Reinstalación'},

  ];
  //INICIALIZACION DE LISTADO DE TIPOS DE TRABAJO

  // DATALIST

  personal_list: any = [];
  sede_list: any = [];
  filteredOptions: any[];
  cards:  { id: number; titulo: string; codigo: string}[] = [];
  idBotones: number = 0;
  indexPasaje : number = 0 ;
  labelText: string = ''; // Texto que se muestra inicialmente
  isFading: boolean = false; // Controla si la animación de desvanecimiento debe activarse
  activarPasajes : boolean = false;
  lugarDestinoPasaje : string = '';

  columnasPasaje: string[] = ['lugarOrigen', 'lugarDestino', 'monto', 'fecha','accion'];
  dataSource: interDetallePasaje[] = [];
  botonesEvidencia: any = {
    botonProblema : { texto : 'EVIDENCIA DEL PROBLEMA' , estado : false},
    botonSolucion : { texto : 'EVIDENCIA DE SOLUCIÓN' , estado : false}
  }

  minFechaLimite: Date | null = new Date(); // 10 de septiembre de 2024 (meses empiezan en 0)
  maxFechaLimite: Date | null = new Date();

  // DATALIST

  // GROUPO DE FORMULARIO

  datos_personal = new FormGroup({
     tecnicoPersonal : new FormControl(null ,Validators.required),
     sedePersonal : new FormControl(null,Validators.required),
     areaPersonal : new FormControl<String | null>(null ,Validators.required),
     minFechaTrabajo : new FormControl<Date | null>(null, Validators.required),
     maxFechaTrabajo : new FormControl<Date | null>(null, Validators.required),
  });

  datos_servicio = new FormGroup({
    descripcionServicio : new FormControl(null ,Validators.required),
    causaServicio : new FormControl(null ,Validators.required),
    selectedOption : new FormControl<String | null>(null ,[Validators.maxLength(12)]),
  });

  datos_trabajo = new FormGroup({
    tipoTrabajo : new FormControl(null ,Validators.required),
    descripcionTrabajo : new FormControl(null ,Validators.required),
  });

  datos_pasajeAcumulado = new FormGroup<any>({
    pasajeTotal: new FormControl<number | null>(null, Validators.required)
  });

  datos_lugarOrigen : FormGroup = new FormGroup<any>({}) ;

  datos_fechaPasaje : FormGroup = new FormGroup<any>({}) ;


  // GROUPO DE FORMULARIO

  constructor(
    private conformidadHelper: ConformidadHelper,
    private renderer: Renderer2,
    private detectorChage : ChangeDetectorRef,
    public dialog: MatDialog,
    private datePipe: DatePipe,
  ) {
    this.filteredOptions = [];
    renderer.setStyle(document.body, 'background', 'var(--light-theme-bg)');
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  accesoDP(){
    return this.datos_personal.controls;
  }

  accesoDS(){
    return this.datos_servicio.controls;
  }

  accesoDT(){
    return this.datos_trabajo.controls;
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  formatoFecha(fecha: Date): string {
    return this.datePipe.transform(fecha, 'dd/MM/yyyy') || '';
  }

  formatoFechaDate(fecha: Date): string {
    return this.datePipe.transform(fecha, 'yyyy/MM/dd') || '';
  }

  async ngOnInit(): Promise<void> {

    const [
      personal,
      sede
    ] = await Promise.all([
      this.conformidadHelper.cargarPersonal(),
      this.conformidadHelper.listadoGeneralHorizon({accion: 1}),
    ]);

    this.personal_list = personal;
    this.sede_list = sede;
    this.dataSource = [];
    this.activarPasajes = false;

    this.habilitarSeccionPersonal(true);

    // Detecta cambios en el formulario de Datos Personal exceptuando 'lugarDestinoPasaje' para desbloquear el boton de 'Agregar Pasaje'
    this.datos_personal.valueChanges.subscribe(value => {

      const controlesInvalidos = Object.keys(this.datos_personal.controls)
        .filter(controlName => this.datos_personal.get(controlName)?.invalid);
      console.log('Controles inválidos con errores:', controlesInvalidos);

      this.activarPasajes = controlesInvalidos.length === 0 ? true : false;

      // En caso ya existan registros en la tabla de pasajes , se limpiará el listado y las filas existentes.
      if(this.dataSource.length > 0 && !this.activarPasajes){
        this.dataSource = [];
        this.indexPasaje = 0;
        this.datos_pasajeAcumulado.reset();
        this.datos_fechaPasaje.reset();
        this.datos_lugarOrigen.reset();
      }

      /*const tecnicoPersonal = this.datos_personal.get('tecnicoPersonal')?.value;
      const sedePersonal = this.datos_personal.get('sedePersonal')?.value;
      const areaPersonal = this.datos_personal.get('areaPersonal')?.value;

      if(tecnicoPersonal && sedePersonal && areaPersonal){
        this.datos_personal.get('lugarDestinoPasaje')?.setErrors({'incorrect': false});
      }else{
        this.datos_personal.get('lugarDestinoPasaje')?.setErrors({'incorrect': true});
      }*/


    });


    // Sincronizar el valor de 'sedePersonal' con los selects la tabla de pasaje
    this.datos_personal.get('sedePersonal')?.valueChanges.subscribe((value) => {
      this.lugarDestinoPasaje = this.sede_list.find((item: any) => item.codlocal === value)?.deslocal || '';
    });

    //Sincroniza todos los inputs de monto de pasaje con el total de pasajes
    this.datos_pasajeAcumulado.valueChanges.subscribe(values => {
      const inputControls = Object.keys(values).filter(key => key.includes('montoPasaje'));
      const sum : number  = (inputControls.reduce((acc, key) => acc + (Number(values[key]) || 0), 0) ) || 0;
      this.datos_pasajeAcumulado.get('pasajeTotal')?.setValue(sum, { emitEvent: false });
    });

    //Condicionar a todos los controladores de Lugar de Origen para que si se escoge el mismo lugar que el lugar de origen salga una advertencia
    this.datos_lugarOrigen.valueChanges.subscribe(value => {
      const lugarDestino = this.datos_personal.get('sedePersonal')?.value;

      if(lugarDestino) {
        const inputControls = Object.keys(value);
        inputControls.forEach((idlugar) => {
          const lugarOrigen = value[idlugar];
          if (lugarOrigen) {
            if (lugarOrigen === lugarDestino) {
              this.datos_lugarOrigen.get(idlugar)?.setErrors({'incorrect': true});
            }
          }else{
            this.datos_lugarOrigen.get(idlugar)?.setErrors({'incorrect': false});
          }
        });
      }else{
        this.datos_personal.get('sedePersonal')?.setErrors({'incorrect': true});
      }

    });

    /*this.datos_fechaTrabajo.get('maxFechaTrabajo')?.valueChanges.subscribe((value) => {
      let minFecha : any = this.datos_fechaTrabajo.get('minFechaTrabajo')?.value;*/
    this.datos_personal.get('maxFechaTrabajo')?.valueChanges.subscribe((value) => {
      let minFecha : any = this.datos_personal.get('minFechaTrabajo')?.value;
      let maxFecha : any = value;

      if( minFecha && maxFecha ){

          this.minFechaLimite = new Date(minFecha);
          this.maxFechaLimite = new Date(maxFecha);

          //Obtener todos los formControNames dentro del formGroup datos_fechaPasaje
          const inputControls = Object.keys(this.datos_fechaPasaje.controls);

          //Iterar sobre cada formControlName
          inputControls.forEach((key) => {
            let fechaPasaje = this.datos_fechaPasaje.get(key)?.value;
            if(fechaPasaje){
              let fechaPasajeDate = new Date(fechaPasaje);
              if(fechaPasajeDate >= (this.minFechaLimite || new Date()) && fechaPasajeDate <= (this.maxFechaLimite|| new Date()) ){
                this.datos_fechaPasaje.get(key)?.setErrors({'incorrect': false});
              }else{
                this.datos_fechaPasaje.get(key)?.setErrors({'incorrect': true});
                this.datos_fechaPasaje.get(key)?.setValue('');
              }
            }
          });
      }
    });
  }

  //AL SELECCIONAR UN REQUERIMIENTO
  onEventSeleccion(event : any){
    const valor = event.option.value;

    if(this.cards.find((item) => item.codigo === valor)){
      this.labelText = 'Ya existe el requerimiento ' + valor + ' en el listado'; // Cambia el texto
      this.isFading = false; // Resetea el estado de opacidad para mostrar el texto

      // Después de 2 segundos, comienza a desvanecer el texto
      setTimeout(() => {
        this.isFading = true;
      }, 2000); // 2000 ms = 2 segundos

    }else{
      this.cards.push({ titulo: `RQ N°${valor}`, id: this.idBotones++ , codigo: valor });
      this.labelText = '';
    }
    this.datos_servicio.get('selectedOption')?.setValue('');
    this.inputBusqueda.nativeElement.blur();
    this.detectorChage.detectChanges();
    this.filteredOptions = [];

  }

  //AL SELECCIONAR INFORMACION DE UN REQUERIMIENTO
  async infoRequerimiento(valor: any){
    const requerimientoInfo = await this.conformidadHelper.listadoGeneralHorizon({accion: 3, nroRequerimiento: valor, sede: ''});
    const dialogRef = this.dialog.open(modalRequerimientoComponent,{
      width: '300rem',
      data: { requerimientoInfo }
    });
    dialogRef.afterClosed().subscribe(result => {
      console.log(`Dialog result: ${result}`);
    });
  }

  //ELIMINAR REQUERIMIENTO SELECCIONADO
  deleteRequerimiento(valor : any){
    this.cards = this.cards.filter((item) => item.id !== valor);
  }

  //AL SUBIR UN ARCHIVO EN LOS BOTONES DE DOCUMENTOS
  onEventoDocumentos(event: any){
    const file = event?.target?.files[0] || null;
    const nameFile = file?.name.replaceAll(' ', '_').toUpperCase() || null;
    const elementId = event.target.getAttribute('id');

    const botonMap = this.botonesEvidencia[elementId];
    console.log(botonMap);
    if(botonMap) {
      if(file){
        botonMap.estado = true;
        botonMap.texto = nameFile;
      }else{
        botonMap.estado = false;
        botonMap.texto = elementId === 'botonProblema' ? 'EVIDENCIA DEL PROBLEMA' : 'EVIDENCIA DE SOLUCIÓN';
      }
    }
  }

  //AL SELECCIONAR UN TECNICO, MOSTRAR AUTOMATICAMENTE SU AREA
  onEventTecnico(event: any) { // IMPRIMER EL ÁREA DEL TÉCNICO AL SELECCIONARLO

    const areaControl = this.accesoDP().areaPersonal;
    const codigoPer = event.value;
    const personal_list = this.personal_list;
    const personal_datos = personal_list.find((item: any) => item.codper === codigoPer);

    if(Object.keys(personal_datos).length  > 0){
      areaControl.setValue(personal_datos.descargo);
    }else{
      areaControl.setValue('Sin Cargo');
    }
    areaControl.updateValueAndValidity();

  }

  //REALIZAR UNA BUSQUEDA DE REQUERIMIENTO
  async onBusquedaRequerimiento(event: any) {
    const codigoSede = this.accesoDP().sedePersonal.value;
    const busquedaReq = event.target.value;

    const jsonParametrosReq = {
      accion : 2,
      nroRequerimiento : busquedaReq,
      sede : codigoSede
    };

    const requerimiento_list = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosReq) ;
    const mapeoResultado = requerimiento_list.map((item: any) => ({
      codigo: item.CODIGONUM,
      serie : item.CSERIE_DOC,
      numero: item.CODIGONUM
    }));

    this.filteredOptions = mapeoResultado;
  }

  //AL SELECCIONAR LA OPCIÓN DE NUEVO PASAJE
  onEventoNuevoPasaje(event : any){
    const NuevaFila : interDetallePasaje = {
      index: this.indexPasaje++,
      lugarOrigen: '',
      lugarDestino: '',
      monto: '',
      fecha: new Date(),
      isNew: true
    };
    this.datos_pasajeAcumulado.addControl('montoPasaje'+NuevaFila.index, new FormControl<number | null>(null,[Validators.maxLength(12)]));
    this.datos_fechaPasaje.addControl('fechaPasaje'+NuevaFila.index, new FormControl<Date | null>(null,[Validators.maxLength(12)]));
    this.datos_lugarOrigen.addControl('lugarOrigen'+NuevaFila.index, new FormControl<String | null>(null ,[Validators.maxLength(12)]));
    this.dataSource = [...this.dataSource,NuevaFila];

    this.activarPasajes = false;
    this.habilitarSeccionPersonal(false);

  }

  onEventoConfirmarPasaje(event: any, elemento: any){
    const lugarOrigen = this.datos_lugarOrigen.get(`lugarOrigen${elemento.index}`)?.value;
    const lugarDestino = this.datos_personal.get('sedePersonal')?.value || '';
    const pasaje = this.datos_pasajeAcumulado.get(`montoPasaje${elemento.index}`)?.value;
    const fechaPasaje = this.datos_fechaPasaje.get(`fechaPasaje${elemento.index}`)?.value;

    //VALIDAR LOS VALORES OBTENIDOS ANTES DE REALIZAR UN REGISTRO


    // Agregar los datos recogidos de la fila para agregarlos en dataSource y actualizar la tabla
    const index = this.dataSource.findIndex((item) => item.index === elemento.index);
    this.dataSource[index] = {
      index: elemento.index,
      lugarOrigen: this.sede_list.find((item: any) => item.codlocal === lugarOrigen)?.deslocal || '' ,
      lugarDestino: this.sede_list.find((item: any) => item.codlocal === lugarDestino)?.deslocal || '' ,
      monto: pasaje,
      fecha: fechaPasaje,
      isNew: false
    };

    console.log(this.dataSource , 'DATASOURCE');
    console.log(lugarOrigen , lugarDestino, pasaje, fechaPasaje)

    this.dataSource = [...this.dataSource];
    this.activarPasajes = true;
    this.habilitarSeccionPersonal(true);

  }

  onEventoCancelarPasaje(event: any, elemento: any){
    this.dataSource = this.dataSource.filter((item) => item.index !== elemento.index);
    this.datos_pasajeAcumulado.removeControl('montoPasaje'+elemento.index);
    this.datos_fechaPasaje.removeControl('fechaPasaje'+elemento.index);
    this.datos_lugarOrigen.removeControl('lugarOrigen'+elemento.index);

    this.habilitarSeccionPersonal(true);
    this.activarPasajes = true
  }


  onMaterialSobrante(event: any){
    const materialSobrante = event.target.value;
    console.log(materialSobrante);
  }

  onEventoEditarPasaje(event: any, elemento: any){

  }

  onEventoEliminarPasaje(event: any, elemento: any){

  }

  habilitarSeccionPersonal(datos: boolean){
    if(datos){
      this.datos_personal.get('tecnicoPersonal')?.enable({ emitEvent: false });
      this.datos_personal.get('sedePersonal')?.enable({ emitEvent: false });
      this.datos_personal.get('areaPersonal')?.enable({ emitEvent: false });
      this.datos_personal.get('maxFechaTrabajo')?.enable({ emitEvent: false });
      this.datos_personal.get('minFechaTrabajo')?.enable({ emitEvent: false });
    }else{
      this.datos_personal.get('tecnicoPersonal')?.disable({ emitEvent: false });
      this.datos_personal.get('sedePersonal')?.disable({ emitEvent: false });
      this.datos_personal.get('areaPersonal')?.disable({ emitEvent: false });
      this.datos_personal.get('maxFechaTrabajo')?.disable({ emitEvent: false });
      this.datos_personal.get('minFechaTrabajo')?.disable({ emitEvent: false });
    }
  }

}

